<?php
// LICENSE
/*
DiggerDiff v1.0
Copyright (c) 2003, Dave Walton
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
// END LICENSE


function action_diff($site, $view, $config) {
  $mode = $config['diffmode'];
  $oldfile = $config['oldpath'] . "/$site.html";
  $newfile = $config['newpath'] . "/$site.html";
  $diffcmd = $config['diffcmd'] . " $oldfile $newfile";

  $sitedata = load_urls($config['urlfile'], $config['sorturls']);
  foreach ($sitedata as $data) {
    if ($data['md5'] === $site) {
      break;
    }
  }

  $output = '';
  $urldata = parse_url($data['url']);
  $base = $urldata['scheme'] . '://' . $urldata['host'];
  $base = (empty($urldata['port'])) ? $base : "$base:" . $urldata['port'];
  $base .= $urldata['path'];
  // The Rendered View actually cannot be validated as any particular
  // standard, since it embeds a diff of unknown code.
  $output .= print_header('trans', '- View Diff - '.$data['title'], "  <base href=\"$base\" target=\"diggerdiff\" />");
  $output .= "<body>\n";

  if (file_exists($oldfile) && file_exists($newfile)) {
    $diff = `$diffcmd`;

    $diff = explode("\n", $diff);
    foreach ($diff as $key => $line) {
      $line = trim($line);
      if (empty($line) || ($line === '+') || ($line === '-')) {
        // strip out blank lines and blank diffs
        unset($diff[$key]);
      } else {
        // Only show lines that start with a char in 'diffshow'
        if (strpos($config['diffshow'], $line[0]) === FALSE) {
	  unset($diff[$key]);
	} else {
          $diff[$key] = $line;
	}
      }
    }
    if ($view === 'source') {
      $sourcediff = "<pre>\n" . htmlspecialchars(implode("\n", $diff)) . "\n</pre>";
    } else {
      if ($mode > 1) {
        $renderdiff = "<pre>\n" . implode("\n", $diff) . "\n</pre>";
      } else {
        $renderdiff = implode("<br />\n", $diff);
      }
    }

    $output .= "<h1>Changes for " . $data['title'] . "</h1>\n\n";
    if ($view === 'source') {
      $output .= "<h2>Source View</h2>\n";
      $output .= "$sourcediff\n\n";
    } else {
      $output .= "<h2>Rendered View</h2>\n";
      $output .= "$renderdiff\n\n";
    }
  }

  $output .= "</body>\n";
  $output .= "</html>\n";

  return($output);
}

?>
