<?php
// LICENSE
/*
DiggerDiff v1.0
Copyright (c) 2003, Dave Walton
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
// END LICENSE


function action_list($page, $config) {
  $output = '';
  // The ONLY reason this is Transitional instead of Strict is that
  // Strict doesn't allow the 'target' attribute of the 'a' tag.
  // (Of course, use of '&' instead of '&amp;' in links in the URL
  // list are an issue for both standards.)
  $output .= print_header('trans', "- View List - Page $page");
  $output .= "<body>\n";
  $sitedata = load_urls($config['urlfile'], $config['sorturls']);
  $pagedata = calc_page($page, $config['perpage'], count($sitedata));
  if ($config['updatemode'] === 'demand') {
    $output .= fetch_pages($pagedata['firstkey'], $pagedata['lastkey'], $config, $sitedata);
  }
  $output .= print_page($config, $sitedata, $pagedata);
  $output .= print_ranges($config['perpage'], $sitedata, $pagedata);
  $output .= "</body>\n";
  $output .= "</html>\n";

  return($output);
}


function calc_page($page, $perpage, $maxkey) {
  $pagedata = array();
  $maxkey--;
  $pagedata['maxkey'] = $maxkey;
  $pagedata['lastpage'] = (int)($maxkey / $perpage) + 1;
  $pagedata['page'] = min($pagedata['lastpage'], max(1, $page));
  $pagedata['firstkey'] = ($pagedata['page'] - 1) * $perpage;
  $pagedata['lastkey'] = min($maxkey, ($pagedata['page'] * $perpage) - 1);

  return($pagedata);
}


function print_list($config, $sitedata, $pagedata) {
  global $PHP_SELF;

  $output = '';
  $output .= "  <tr>\n";
  foreach ($config['columns'] as $column) {
    switch ($column) {
      case 'archive':
        $output .= "    <th>Archive</th>\n";
        break;
      case 'blank':
        $output .= "    <th>&nbsp;</th>\n";
        break;
      case 'current':
        $output .= "    <th>Current</th>\n";
        break;
      case 'edit':
        $output .= "    <th>Edit</th>\n";
        break;
      case 'fetch':
        $output .= "    <th>Fetch</th>\n";
        break;
      case 'save':
        $output .= "    <th>Save</th>\n";
	break;
      case 'site':
        $output .= "    <th>Site</th>\n";
        break;
      case 'status':
        $output .= "    <th>Status</th>\n";
        break;
      default:
        $output .= "    <th>$column</th>\n";
	break;
    }
  }
  $output .= "  </tr>\n";

  $page = $pagedata['page'];
  // TODO:  This $revcols stuff doesn't adequately handle multiple instances
  $revcols = array_flip($config['columns']);
  $arcloc = isset($revcols['archive']) ? $revcols['archive'] : -1;
  $curloc = isset($revcols['current']) ? $revcols['current'] : -1;
  $savloc = isset($revcols['save']) ? $revcols['save'] : -1;
  $siteloc = isset($revcols['site']) ? $revcols['site'] : -1;
  $fetchloc = isset($revcols['fetch']) ? $revcols['fetch'] : -1;
  for ($key=$pagedata['firstkey']; $key<=$pagedata['lastkey']; $key++) {
    $url = $sitedata[$key]['url'];
    $title = $sitedata[$key]['title'];
    $urlmd5 = $sitedata[$key]['md5'];
    $postdata = $sitedata[$key]['postdata'];
    $oldfile = $config['oldpath'] . "/$urlmd5.html";
    $newfile = $config['newpath'] . "/$urlmd5.html";

    if (!file_exists($newfile)) {
      $output .= fetch_pages($key, $key, $config, $sitedata);
    }
    $newcontents = @file($newfile);
    $newcontents = implode('', $newcontents);
    $newtime = filemtime($newfile);
    $newmd5 = md5($newcontents);

    if (file_exists($oldfile)) {
      $oldcontents = @file($oldfile);
      $oldcontents = implode('', $oldcontents);
      $oldtime = filemtime($oldfile);
    } else {
      $oldcontents = '';
    }
    $oldmd5 = md5($oldcontents);

    if (empty($newcontents)) {
      $status = 'ERROR';
    } else if (empty($oldcontents)) {
      $status = 'NEW';
    } else if ($newmd5 !== $oldmd5) {
      $status = 'CHANGED';
    } else {
      $status = 'SAME';
    }

    $output .= "  <tr>\n";

    foreach ($config['columns'] as $column) {
      switch ($column) {
        case 'archive':
          $output .= "    <td align=\"center\">";
          if (empty($oldcontents)) {
            $output .= '&nbsp;';
          } else {
            $output .= "<a href=\"$PHP_SELF?action=view&amp;version=old&amp;site=$urlmd5\" target=\"diggerdiff\">";
            $output .= date("d M Y", $oldtime) . "</a>";
          }
          $output .= "</td>\n";
          break;
	case 'blank':
	  $output .= "    <td>&nbsp;</td>\n";
	  break;
        case 'current':
          $output .= '    <td align="center">';
          if (empty($newcontents)) {
            $output .= '&nbsp;';
          } else {
            $output .= "<a href=\"$PHP_SELF?action=view&amp;version=new&amp;site=$urlmd5\" target=\"diggerdiff\">View</a>";
          }
          $output .= "</td>\n";
          break;
	case 'edit':
	  $output .= '    <td align="center">';
	  $output .= "<a href=\"$PHP_SELF?action=edit&amp;&amp;mode=Edit+Line&amp;site=$urlmd5&amp;page=$page\">Edit</a>";
          $output .= "</td>\n";
          break;
	case 'fetch':
	  $output .= '    <td align="center">';
          if (($status !== 'ERROR') && ($newtime > (time() - $config['agelimit']))) {
	    $output .= '&nbsp;';
	  } else {
            $output .= "<a href=\"$PHP_SELF?action=fetch&amp;site=$urlmd5&amp;page=$page\">";
	    if (($siteloc >= 0) && ($curloc >= 0)) {
	      if (($curloc > $fetchloc) && ($fetchloc > $siteloc)) {
	        $output .= '&gt;&gt;&gt;';
	      } else if (($curloc < $fetchloc) && ($fetchloc < $siteloc)) {
	        $output .= '&lt;&lt;&lt;';
	      } else {
	        $output .= 'Fetch';
	      }
	    } else {
	      $output .= 'Fetch';
	    }
	    $output .= "</a>";
	  }
	  $output .= "</td>\n";
	  break;
	case 'save':
	  $output .= '    <td align="center">';
	  if (($status === 'SAME') || ($status === 'ERROR')) {
	    $output .= '&nbsp;';
	  } else {
            $output .= "<a href=\"$PHP_SELF?action=save&amp;site=$urlmd5&amp;page=$page\">";
	    if (($arcloc >= 0) && ($curloc >= 0)) {
	      if (($arcloc > $savloc) && ($savloc > $curloc)) {
	        $output .= '&gt;&gt;&gt;';
	      } else if (($arcloc < $savloc) && ($savloc < $curloc)) {
	        $output .= '&lt;&lt;&lt;';
	      } else {
	        $output .= 'Save';
	      }
	    } else {
	      $output .= 'Save';
	    }
	    $output .= "</a>";
	  }
	  $output .= "</td>\n";
	  break;
        case 'site':
          if (empty($postdata)) {
            $output .= "    <td><a href=\"$url\" target=\"diggerdiff\">$title</a></td>\n";
          } else {
            $output .= "    <td><form method=\"post\" action=\"$url\" target=\"diggerdiff\">\n";
            $output .= "        <div>\n";
            $postdata = explode('&', $postdata);
            foreach ($postdata as $post) {
              $pos = strpos($post, '=');
	      if ($pos === FALSE) {
                $output .= "        <input type=\"hidden\" name=\"$post\" />\n";
	      } else {
	        $output .= "        <input type=\"hidden\" name=\"";
	        $output .= substr($post, 0, $pos) . "\" value=\"";
	        $output .= urldecode(substr($post, $pos+1)) . "\" />\n";
	      }
            }
            $output .= "        <input type=\"submit\" value=\"$title\" />\n";
            $output .= "        </div>\n";
            $output .= "        </form></td>\n";
          }
          break;
        case 'status':
          $output .= "    <td align=\"center\">";
	  switch ($status) {
	    case 'ERROR':
              $output .= '<span style="color:red">ERROR</span>';
	      break;
	    case 'NEW':
              $output .= '<span style="color:black">NEW</span>';
	      break;
	    case 'CHANGED':
              $output .= "<a href=\"$PHP_SELF?action=frame2&amp;site=$urlmd5\" target=\"diggerdiff\">";
              $output .= '<span style="color:red">CHANGED</span></a>';
	      break;
	    case 'SAME':
              $output .= '<span style="color:green">SAME</span>';
	      break;
	    default:
	      $output .= 'unknown';
	      break;
	  }
          $output .= "</td>\n";
          break;
        default:
	  $output .= "    <td>Invalid column ID</td>\n";
	  break;
      }
    }

    $output .= "  </tr>\n";
  }

  return($output);
}


function print_page($config, $sitedata, $pagedata) {
  global $PHP_SELF;

  $output = '';
  $page = $pagedata['page'];
  $numcols = count($config['columns']);
  $output .= "<table border=\"1\">\n";

  foreach ($config['rows'] as $row) {
    switch($row) {
      case 'blank':
        $output .= "  <tr>\n";
        $output .= "    <td colspan=\"$numcols\">&nbsp;</td>\n";
        $output .= "  </tr>\n";
        break;
      case 'edit':
        $output .= "  <tr>\n";
        $output .= "    <td colspan=\"$numcols\" align=\"center\">\n";
        $output .= "      <a href=\"$PHP_SELF?action=edit\" target=\"_blank\">Edit URL list</a>\n";
        $output .= "    </td>\n";
        $output .= "  </tr>\n";
        break;
      case 'frames':
        $output .= "  <tr>\n";
        $output .= "    <td colspan=\"$numcols\" align=\"center\">\n";
        $output .= "      <a href=\"$PHP_SELF?action=list&amp;page=$page\" target=\"_top\">No frames</a>\n";
        $output .= "      &nbsp;&nbsp;&nbsp;\n";
        $output .= "      <a href=\"$PHP_SELF?action=frame1&amp;page=$page\" target=\"_top\">Frames</a>\n";
        $output .= "    </td>\n";
        $output .= "  </tr>\n";
        break;
      case 'list':
        $output .= print_list($config, $sitedata, $pagedata);
        break;
      case 'pages':
        $output .= "  <tr>\n";
        $output .= "    <td colspan=\"$numcols\" align=\"center\">\n";
        if ($page > 1) {
          $output .= "<a href=\"$PHP_SELF?action=list&amp;page=" . ($page-1) . "\">&lt;&lt;</a>&nbsp;&nbsp;";
        } else {
          $output .= "&lt;&lt;&nbsp;&nbsp;";
        }
        for ($pagenum=1; $pagenum<=$pagedata['lastpage']; $pagenum++) {
          if ($pagenum === $page) {
	    $output .= "<b style=\"font-size: larger;\">$pagenum</b>";
	  } else {
            $output .= "<a href=\"$PHP_SELF?action=list&amp;page=$pagenum\">$pagenum</a>";
          }
          $output .= "&nbsp;";
        }
        if ($page < $pagedata['lastpage']) {
          $output .= "&nbsp;<a href=\"$PHP_SELF?action=list&amp;page=" . ($page+1) . "\">&gt;&gt;</a>";
        } else {
          $output .= "&nbsp;&gt;&gt;";
        }
        $output .= "    </td>\n";
        $output .= "  </tr>\n";
        break;
      case 'save':
        $output .= "  <tr>\n";
        $output .= "    <td colspan=\"$numcols\" align=\"center\">\n";
        $output .= "      <a href=\"$PHP_SELF?action=save&amp;page=$page\">Save all NEW/CHANGED to Archive</a>\n";
        $output .= "    </td>\n";
        $output .= "  </tr>\n";
        break;
      default:
        $output .= "  <tr>\n";
        $output .= "    <td colspan=\"$numcols\" align=\"center\">$row</td>\n";
        $output .= "  </tr>\n";
        break;
    }
  }

  $output .= "</table>\n";
  return($output);
}


function print_ranges($perpage, $sitedata, $pagedata) {
  global $PHP_SELF;

  $output = '';
  $output .= "\n<p></p>\n";
  $output .= "<table border=\"1\">\n";
  $output .= "  <tr>\n";
  $output .= "    <th>Page</th>\n";
  $output .= "    <th colspan=\"3\">Range</th>\n";
  $output .= "  </tr>\n";

  for ($page=1; $page<=$pagedata['lastpage']; $page++) {
    $firstkey = ($page - 1) * $perpage;
    $lastkey = min($pagedata['maxkey'], ($page * $perpage) - 1);
    $firsttitle = $sitedata[$firstkey]['title'];
    $lasttitle = $sitedata[$lastkey]['title'];
    if ($page > 1) {
      $prevlasttitle = $sitedata[$firstkey-1]['title'];
    }
    if ($page < $pagedata['lastpage']) {
      $nextfirsttitle = $sitedata[$lastkey+1]['title'];
    }

    $output .= "  <tr>\n";
    $output .= "    <td align=\"right\"><a href=\"$PHP_SELF?action=list&amp;page=$page\">$page</a></td>\n";
    if ($page > 1) {
      if (strcasecmp($prevlasttitle, $firsttitle) === 0) {
        $len = strlen($firsttitle);
      } else {
        $len = 1;
        // strncasecmp in PHP >= 4.0.2
        while (strncmp(strtolower($prevlasttitle), strtolower($firsttitle), $len) === 0) {
          $len++;
        }
      }
      $beginrange = substr($firsttitle, 0, $len);
    } else {
      $beginrange = $firsttitle[0];
    }
    if ($firstkey === $lastkey) {
      $output .= "    <td colspan=\"3\" align=\"center\">";
      $output .= "<a href=\"$PHP_SELF?action=list&amp;page=$page\">$beginrange</a></td>\n";
    } else {
      if (strcasecmp($firsttitle, $lasttitle) === 0) {
        $len = strlen($lasttitle);
      } else {
        $len = 1;
        // strncasecmp in PHP >= 4.0.2
        while (strncmp(strtolower($firsttitle), strtolower($lasttitle), $len) === 0) {
          $len++;
        }
      }
      if ($page < $pagedata['lastpage']) {
        if (strcasecmp($lasttitle, $nextfirsttitle) === 0) {
          $len2 = strlen($lasttitle);
        } else {
          $len2 = 1;
          // strncasecmp in PHP >= 4.0.2
          while (strncmp(strtolower($lasttitle), strtolower($nextfirsttitle), $len2) === 0) {
            $len2++;
          }
        }
        $len = max($len, $len2);
      }
      $endrange = substr($lasttitle, 0, $len);
      $output .= "    <td align=\"right\"><a href=\"$PHP_SELF?action=list&amp;page=$page\">$beginrange</a></td>\n";
      $output .= "    <td>&nbsp;=&gt;&nbsp;</td>\n";
      $output .= "    <td><a href=\"$PHP_SELF?action=list&amp;page=$page\">$endrange</a></td>\n";
    }
    $output .= "  </tr>\n";
  }

  $output .= "</table>\n";
  return($output);
}

?>
