<?php
// LICENSE
/*
DiggerDiff v1.0
Copyright (c) 2003, Dave Walton
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
// END LICENSE


// TODO:  Use curl_*() or PHP 5 to do posts (config option?)
// TODO:  Use curl_*() or PHP to do https (config option?)


require('./config.php');

if ($config['updatemode'] === 'cron') {
  if ($argc > 0) {
    foreach ($argv as $arg) {
      if ($arg === 'cron') {
        set_time_limit(0);
        $action = 'cron';
        break;
      }
    }
  }
}


function save_file($name, $contents) {
  $fp = fopen($name, 'w');
  fwrite($fp, $contents);
  fclose($fp);
}


// Used by: action_cron, action_diff, action_save
function load_urls($urlfile, $sorturls) {
  $sitedata = array();

  $urllist = file($urlfile);
  if ($sorturls === TRUE) {
    natcasesort($urllist);
  }
  reset($urllist);

  $index = 0;
  foreach ($urllist as $url) {
    $url = trim($url);
    if (empty($url) || ($url[0] === '#')) {
      continue;
    }
    $sitedata[$index] = parse_line($url);
    if (empty($sitedata[$index]['title'])) {
      $sitedata[$index]['title'] = $sitedata[$index]['url'];
    }
    $index++;
  }

  return($sitedata);
}


// Used by: action_cron, action_list
function fetch_pages($firstkey, $lastkey, $config, $sitedata) {
  $output = '';
  for ($key=$firstkey; $key<=$lastkey; $key++) {
    $now = time();
    $url = $sitedata[$key]['url'];
    $title = $sitedata[$key]['title'];
    $urlmd5 = $sitedata[$key]['md5'];
    $postdata = $sitedata[$key]['postdata'];
    $regexes = $sitedata[$key]['regex'];
    $newfile = $config['newpath'] . "/$urlmd5.html";

    $newfiletime = @filemtime($newfile);
    if ($newfiletime !== FALSE) {
      $currentcontents = @file($newfile);
      if (!empty($currentcontents) && ($newfiletime > ($now - $config['agelimit']))) {
        continue;
      }
    }

    if ($config['updatemode'] === 'cron') {
      $output .= "Fetching [$title]... ";
    }

    if (!empty($postdata)) {
      $newcontents = shell_exec($config['postcmd'] . " '$postdata' '$url'");
      if (empty($newcontents)) {
        $newcontents = FALSE;
      } else {
        // put into an array for consistency with file()
        $newcontents = array($newcontents);
      }
    // use parse_url() here??
    } else if (strtolower(substr($url, 0, 8)) === 'https://') {
      // For PHP >= 4.0.3
      //$newcontents = shell_exec($config['httpscmd'] . " " . escapeshellarg($url));
      $newcontents = shell_exec($config['httpscmd'] . " '$url'");
      if (empty($newcontents)) {
        $newcontents = FALSE;
      } else {
        // put into an array for consistency with file()
        $newcontents = array($newcontents);
      }
    } else {
      // TODO:  This doesn't support redirects, but newer versions of PHP can
      $newcontents = @file($url);
    }

    if ($newcontents === FALSE) {
      $newcontents = '';
    } else {
      $newcontents = implode('', $newcontents);
      // detect line endings and clean up accordingly
      if (strpos($newcontents, "\r\n") !== FALSE) {
        // DOS file
        $newcontents = explode("\r\n", $newcontents);
      } else if (strpos($newcontents, "\n") !== FALSE) {
        // Unix file
        $newcontents = explode("\n", $newcontents);
      } else if (strpos($newcontents, "\r") !== FALSE) {
        // Mac file
        $newcontents = explode("\r", $newcontents);
      } else {
        // Unknown file?
	// Gotta put it back into an array for consistency
	$newcontents = array($newcontents);
      }
      $newcontents = implode("\n", $newcontents);

      // strip out any specified patterns
      foreach ($regexes as $regex) {
	$divider = $regex[0];
        $divcount = substr_count($regex, $divider);
	// MUST be in form "/search/replace/[modifiers]"
	if ($divcount !== 3) {
	  $output .= "(Warning: $divcount dividers in regex [$regex].) ";
	  continue;
	}
	$regparts = explode($divider, $regex);
	// Rearrange to "/search/[modifiers]" and "replace"
	$search = $divider . $regparts[1] . $divider;
	if (!empty($regparts[3])) {
	  // disallow the 'e' modifier for security reasons
	  $search .= str_replace('e', '', $regparts[3]);
	}
	$replace = empty($regparts[2]) ? '' : stripcslashes($regparts[2]);
	$newcontents = preg_replace($search, $replace, $newcontents);
      }
    }

    if (!file_exists($newfile) || (md5($newcontents) !== md5($currentcontents))) {
      save_file($newfile, $newcontents);
    }

    if ($config['updatemode'] === 'cron') {
      if (empty($newcontents)) {
        $output .= "ERROR\n";
      } else {
        $output .= "Success\n";
      }
    }
  }
  return($output);
}


// Used by: load_urls, action_edit
function parse_line($line) {
  $linedata = explode(chr(0x09), $line);
  $fields = count($linedata);
  $title = '';
  $url = '';
  $postdata = '';
  $regexes = array();
  $md5 = '';

  if (empty($line)) {
    // Format: blank
  } else if ($line[0] === '#') {
    // Format: #Comment
    $title = $line;
  } else {
    if (isset($linedata[2]) && (strtoupper(substr($linedata[2], 0, 5)) === 'POST ')) {
      // Format: Title<TAB>URL<TAB>POST PostData[<TAB>Regex]...
      $title = $linedata[0];
      $url = $linedata[1];
      $postdata = substr($linedata[2], 5);
      for ($field=3; $field<$fields; $field++) {
        $regexes[] = $linedata[$field];
      }
    } else if (isset($linedata[1])) {
      // Format: Title<TAB>URL[<TAB>Regex]...
      $title = $linedata[0];
      $url = $linedata[1];
      for ($field=2; $field<$fields; $field++) {
        $regexes[] = $linedata[$field];
      }
    } else {
      // Format: URL
      $url = $linedata[0];
    }
    $md5 = md5($url . $postdata);
  }

  $linedata = array();
  $linedata['title'] = $title;
  $linedata['url'] = $url;
  $linedata['postdata'] = $postdata;
  $linedata['regex'] = $regexes;
  $linedata['md5'] = $md5;
  return($linedata);
}


// Used by: action_diff, action_frame, action_save, action_view
function print_header($style = 'trans', $title = '', $head = '') {
  $output = '';
  $output .= "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
  switch ($style) {
    case 'frame':
      $output .= "<!DOCTYPE html\n";
      $output .= "   PUBLIC \"-//W3C//DTD XHTML 1.0 Frameset//EN\"\n";
      $output .= "   \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd\">\n";
      break;
    case 'strict':
      $output .= "<!DOCTYPE html\n";
      $output .= "   PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n";
      $output .= "   \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
      break;
    case 'trans':
    default:
      $output .= "<!DOCTYPE html\n";
      $output .= "   PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n";
      $output .= "   \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
      break;
  }
  $output .= "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n";
  $output .= "<head>\n";
  $output .= "  <title>DiggerDiff $title</title>\n";
  $output .= "$head\n";
  $output .= "</head>\n\n";

  return($output);
}


// ==================================================


// Initialize input variables
// What action to perform
$action = empty($action) ? $config['defaultaction'] : $action;
// Line to edit (edit)
$line = empty($line) ? 0 : (int)$line;
// Edited line data (edit)
$linedata = isset($linedata) ? $linedata : array();
// Edit mode (edit)
$mode = empty($mode) ? '' : $mode;
// Current page (edit, fetch, frame1, list, save)
$page = empty($page) ? 1 : (int)$page;
// MD5 of URL to operate on (diff, edit, fetch, frame2, save, view)
$site = empty($site) ? '' : $site;
$site = ereg_replace("[^[:alnum:]]", '', $site);
// Old or New version (view)
$version = empty($version) ? '' : strtolower($version);
// Rendered or Source view (diff)
$view = empty($view) ? 'render' : strtolower($view);

switch ($action) {
  case 'cron':
    include_once('./dd_cron.php');
    action_cron($config);
    break;
  case 'diff':
    include_once('./dd_diff.php');
    echo action_diff($site, $view, $config);
    break;
  case 'edit':
    include_once('./dd_edit.php');
    action_edit($mode, $line, $linedata, $page, $site, $config);
    break;
  case 'fetch':
    include_once('./dd_fetch.php');
    action_fetch($site, $page, $config);
    break;
  case 'frame1':
  case 'frame2':
    include_once('./dd_frame.php');
    action_frame($action, $site, $page, $config);
    break;
  case 'list':
    include_once('./dd_list.php');
    echo action_list($page, $config);
    break;
  case 'save':
    include_once('./dd_save.php');
    echo action_save($page, $config, $site);
    break;
  case 'view':
    include_once('./dd_view.php');
    action_view($site, $version, $config);
    break;
  default:
    echo print_header('strict');
    echo "<body>\n";
    echo "<h1>Invalid action.</h1>\n";
    echo "</body>\n\n";
    echo "</html>\n";
    break;
}

exit;

?>
